#include <sys/types.h>
#include <sys/ticlts.h>
#include <fcntl.h>
#include <stdio.h>
#include <tiuser.h>

#define LOOPADDR "example.addr"

main(argc, argv)
int argc;
char *argv[];
{
   struct localinfo {
       struct tcl_opt_hdr header;
       struct tcl_opt_uid data;
   }  *option_datap;

   int uid;                  /* user ID of sender    */
   int addrlen;              /* used for t_bind()    */
   int flags;                /* used in t_rcvudata() */
   int fd;                   /* transport endpoint   */
   struct t_unitdata *datap; /* data in datagram     */
   struct t_bind *req, *ret; /* used in binding      */

   /*
    *  Create a transport endpoint into the SVR4
    *  connectionless loopback transport provider.
    */

   if ((fd = t_open("/dev/ticlts", O_RDWR, 0)) < 0) {
      t_error("t_open");
      exit(1);
   }

   /*
    *  Allocate space to hold the addresses...
    */

   if ((req = (struct t_bind *)t_alloc(fd, T_BIND, T_ALL))
                                                  == NULL) {
      t_error("t_alloc of request addr failed");
      exit(1);
   }

   if ((ret = (struct t_bind *)t_alloc(fd, T_BIND, T_ALL))
                                                 == NULL) {
      t_error("t_alloc of return addr failed");
      exit(1);
   }

   addrlen = strlen(LOOPADDR) + 1;
   req->qlen = 0;
   req->addr.len = addrlen;
   strncpy(req->addr.buf, LOOPADDR, addrlen);

   /*
    *  Bind to the address and check that we bound correctly
    */

   if (t_bind(fd, req, ret) < 0) {
      t_error("t_bind");
      exit(1);
   }

   if ((req->addr.len != ret->addr.len) ||
     (memcmp(req->addr.buf, ret->addr.buf, req->addr.len))){
      fprintf(stderr, "Did not bind to correct address!\n");
      exit(1);
   }

   /*
    *  Set the "user-info" option...
    */

   if (set_user_info_opt(fd) < 0) {
       return(-1);
   }

   /*
    *  Allocate space to hold a datagram and wait for a
    *  datagram to arrive.
    */

   if ((datap = (struct t_unitdata *)t_alloc(fd, T_UNITDATA,
                                          T_ALL)) == NULL) {
      t_error("t_alloc, ret");
      exit(1);
   }

   if (t_rcvudata(fd, datap, &flags) < 0) {
       t_error("could not read datagram!");
       exit(1);
   }

   /*
    *  Now, extract the user ID from the datagram.  The
    *  transport provider put the user ID in the opt
    *  field because we set the user-info option.
    */

   option_datap = (struct localinfo *)(datap->opt.buf);
   if (option_datap->data.uid_type != TCL_OPT_UID) {
       (void)t_free((char *)datap, T_UNITDATA);
       exit(1);
   }

   uid = option_datap->data.uid_val;

   printf("uid = %d\n", uid);
}
