#include <tiuser.h>
#include <stropts.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

extern int t_errno;

#define SERV_PORT 5134

main()
{
   int listen_fd;       /* file descriptor into transport */
   int recfd;           /* fd on which to to accept       */
   struct t_bind *req;  /* address to which to bind       */
   struct t_bind *ret;  /* actual address that was bound  */
   struct t_call *call; /* used to listen for connection  */
   struct sockaddr_in myaddr; /* holds the address of     */
                              /* this service             */
   if ((listen_fd = t_open("/dev/tcp", O_RDWR, NULL)) < 0) {
      t_error("t_open failed");
      exit(1);
   }

   /*
    *   Get space to hold the address to which you
    *   want to bind
    */

   if ((req = (struct t_bind *)t_alloc(listen_fd, T_BIND,
                                T_ALL)) == NULL) {
      t_error("t_alloc failed");
      exit(1);
   }

   /*
    *   Get space to hold the address o which
    *   you actually bound
    */

   if ((ret = (struct t_bind *)t_alloc(listen_fd, T_BIND,
                               T_ALL)) == NULL) {
      t_error("t_alloc failed");
      exit(1);
   }

   /*
    *  Set qlen to 1 and fill in the address.
    */

   req->qlen = 1;
   req->addr.len = sizeof(struct sockaddr_in);

   memset((void *)&myaddr, 0, sizeof(struct sockaddr_in));
   myaddr.sin_family = AF_INET;
   myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
   myaddr.sin_port = htons(SERV_PORT);

   (void)memcpy(req->addr.buf, &myaddr,
                                sizeof(struct sockaddr_in));

   /*
    *  Bind to the address the service will be offered
    *  over.
    */

   if (t_bind(listen_fd, req, ret) < 0) {
      t_error("t_bind failed");
      exit(1);
   }

   /*
    *  Make sure the address you got was the one you wanted!
    */

   if ((req->addr.len != ret->addr.len) ||
     (memcmp(req->addr.buf, ret->addr.buf, req->addr.len))){
      fprintf(stderr, "Did not bind to correct address!\n");
      exit(1);
   }
   if ((call = (struct t_call *)t_alloc(listen_fd, T_CALL,
                                 T_ALL)) == NULL) {
      t_error("could not allocate space for t_call!");
      exit(1);
   }
   /*
    *  Loop continuously, listening for connection requests,
    *  accepting the call, and performing the service.
    */

   while (1) {
      if (t_listen(listen_fd, call) < 0) {
         t_error("could not listen!");
         exit(1);
      }

      if ((recfd = t_open("/dev/tcp", O_RDWR, NULL)) < 0) {
         t_error("could not open device!");
         exit(1);
      }

      if (t_bind(recfd, NULL, NULL) < 0) {
         t_error("bind for responding failed!");
         exit(1);
      }

      if (t_accept(listen_fd, recfd, call) < 0) {

         /*
          *  Did it fail because the client disconnected?
          *  t_errno is set to TLOOK if it did.
          */

         if (t_errno == TLOOK) {
            if (t_rcvdis(listen_fd, NULL) < 0) {
               t_error("could not disconnect");
               exit(1);
            }
            (void)t_close(recfd);
            continue;
         } else {
            t_error("could not accept call");
            exit(1);
         }
      }
      switch (fork()) {
         case -1:
            perror("fork failed!\n");
            exit(1);

         default: /* parent */

            t_close(recfd);
            break;

         case 0:  /* child */

            t_close(listen_fd);

            /*
             *  The child_actions() routine exits
             *  when it completes
             */
            child_actions(recfd);
      }
   }
}
int
child_actions(resfd)
int resfd;
{
   while (perform_actions(resfd))
                ;

   /*
    *  Did we exit the above loop because the orderly
    *  release happened?  If so, accept the orderly
    *  release and send an acknowledgment.
    */

   if (t_look(resfd) == T_ORDREL) {
      if (t_rcvrel(resfd) < 0) {
         t_error("t_rcvrel failed!");
         exit(1);
      }
      if (t_sndrel(resfd) < 0) {
         t_error("t_sndrel failed!");
         exit(1);
      }
      exit(0);
   }

   exit(1);
}
int
perform_actions(conn_fd)
int conn_fd;
{
   int done;      /* indicates all data are read       */
   int where;     /* points to where we are in buffer  */
   int nbytes;    /* the number of bytes read          */
   int flags = 0; /* info returned from t_rcv()        */
   char reply;    /* a 'y' or 'n' to reply to client   */
   char buf[BUFSIZ];   /* buffer to hold the name      */

   char cmd[BUFSIZ];     /* used to figure out if the  */
   char junkbuf[BUFSIZ]; /* specified user is logged   */
   FILE *fp;             /* onto the system            */

   /*
    *  Read the user name from the client.
    *  The flags argument would contain information like
    *  whether the data was "expedited", but it is ignored
    *  in this example.
    */

   done = 0;
   where = 0;
   do {
      if ((BUFSIZ - where) == 0) {
         return(0);
      }
      if ((nbytes = t_rcv(conn_fd, &buf[where],
                  BUFSIZ - where, &flags)) < 0) {
         return(0);
      }
      where += nbytes;
      if (buf[where - 1] == '\0') {
         done = 1;
      }
   } while (!done);

   /*
    *  Determine if the user is logged on...
    */

   sprintf(cmd, "who | grep '^%s '", buf);
   if ((fp = popen(cmd, "r")) == NULL
    || fgets(junkbuf, BUFSIZ, fp) == NULL) {
      reply = 'n';
   } else {
      reply = 'y';
   }

   /*
    *  Send a "y" or an "n" to the client.
    *  The last argument is 0, specifying that no
    *  flags are sent.
    */

   if (t_snd(conn_fd, &reply, 1, 0) < 0) {
      t_error("t_snd failed!");
      return(0);
   }
   return(1);
}
